@@ -41,13 +41,13 @@
 #include "panel-config-global.h"
 #include "panel-stock-icons.h"
 #include "panel-action-button.h"
-#include "panel-recent.h"
 #include "panel-profile.h"
 #include "panel-menu-button.h"
 #include "panel-menu-items.h"
 #include "panel-globals.h"
 #include "panel-run-dialog.h"
 #include "panel-lockdown.h"
+#include "panel-icon-names.h"
 
 typedef struct {
 	GtkWidget    *pixmap;
@@ -216,10 +216,6 @@ menus_have_icons_changed (GConfClient *client,
 		if (!image)
 			continue;
 
-		/* A forced image is always on */
-		if (g_object_get_data (G_OBJECT (item), "Panel:ForceImage"))
-			continue;
-
 		cur_image = gtk_image_menu_item_get_image (
 					GTK_IMAGE_MENU_ITEM (item));
 
@@ -640,7 +636,7 @@ add_app_to_desktop (GtkWidget      *item,
 	if (desktop_is_home_dir ()) {
 		target_dir = g_build_filename (g_get_home_dir (), NULL);
 	} else {
-		target_dir = g_build_filename (g_get_home_dir (), "Desktop", NULL);
+		target_dir = g_strdup (g_get_user_special_dir (G_USER_DIRECTORY_DESKTOP));
 	}
 
 	target_uri = panel_make_unique_desktop_uri (target_dir, source_uri);
@@ -884,6 +880,14 @@ restore_grabs(GtkWidget *w, gpointer data)
 	gtk_grab_add (GTK_WIDGET (menu));
 }
 
+static void
+menu_destroy_context_menu (GtkWidget *item,
+			   GtkWidget *menu)
+{
+	g_signal_handlers_disconnect_by_func (menu, restore_grabs, item);
+	gtk_widget_destroy (menu);
+}
+
 static GtkWidget *
 create_item_context_menu (GtkWidget   *item,
 			  PanelWidget *panel_widget)
@@ -918,75 +922,51 @@ create_item_context_menu (GtkWidget   *item,
 		return NULL;
 
 	menu = create_empty_menu ();
-	g_object_set_data_full (G_OBJECT (item),
-				"panel-item-context-menu",
-				g_object_ref (menu),
-				(GDestroyNotify) g_object_unref);
-	gtk_object_sink (GTK_OBJECT (menu));
-
+	g_object_set_data (G_OBJECT (item), "panel-item-context-menu", menu);
 	g_object_set_data (G_OBJECT (menu), "menu_panel", panel_widget);
+
+	g_signal_connect (item, "destroy",
+			  G_CALLBACK (menu_destroy_context_menu), menu);
 	g_signal_connect (menu, "deactivate",
 			  G_CALLBACK (restore_grabs), item);
 
-	menuitem = gtk_image_menu_item_new ();
-	setup_menuitem (menuitem,
-			panel_menu_icon_get_size (),
-			NULL,
-			_("Add this launcher to _panel"),
-			FALSE);
+	menuitem = gtk_menu_item_new_with_mnemonic (_("Add this launcher to _panel"));
 	g_signal_connect (menuitem, "activate",
 			  G_CALLBACK (add_app_to_panel), entry);
 	gtk_widget_set_sensitive (menuitem, id_lists_writable);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
+	gtk_widget_show (menuitem);
 
-	menuitem = gtk_image_menu_item_new ();
-	setup_menuitem (menuitem,
-			panel_menu_icon_get_size (),
-			NULL,
-			_("Add this launcher to _desktop"),
-			FALSE);
+	menuitem = gtk_menu_item_new_with_mnemonic (_("Add this launcher to _desktop"));
 	g_signal_connect (menuitem, "activate",
 			  G_CALLBACK (add_app_to_desktop), entry);
 	gtk_widget_set_sensitive (menuitem, id_lists_writable);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
+	gtk_widget_show (menuitem);
 
 
 	submenu = create_empty_menu ();
 
 	g_object_set_data (G_OBJECT (submenu), "menu_panel", panel_widget);
 
-	menuitem = gtk_image_menu_item_new ();
-	setup_menuitem (menuitem,
-			panel_menu_icon_get_size (),
-			NULL,
-			_("_Entire menu"),
-			FALSE);
+	menuitem = gtk_menu_item_new_with_mnemonic (_("_Entire menu"));
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), submenu);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
+	gtk_widget_show (menuitem);
 
-
-	menuitem = gtk_image_menu_item_new ();
-	setup_menuitem (menuitem,
-			panel_menu_icon_get_size (),
-			NULL,
-			_("Add this as _drawer to panel"),
-			FALSE);
+	menuitem = gtk_menu_item_new_with_mnemonic (_("Add this as _drawer to panel"));
 	g_signal_connect (menuitem, "activate",
 			  G_CALLBACK (add_menudrawer_to_panel), entry);
 	gtk_widget_set_sensitive (menuitem, id_lists_writable);
 	gtk_menu_shell_append (GTK_MENU_SHELL (submenu), menuitem);
+	gtk_widget_show (menuitem);
 
-
-	menuitem = gtk_image_menu_item_new ();
-	setup_menuitem (menuitem,
-			panel_menu_icon_get_size (),
-			NULL,
-			_("Add this as _menu to panel"),
-			FALSE);
+	menuitem = gtk_menu_item_new_with_mnemonic (_("Add this as _menu to panel"));
 	g_signal_connect (menuitem, "activate",
 			  G_CALLBACK (add_menu_to_panel), entry);
 	gtk_widget_set_sensitive (menuitem, id_lists_writable);
 	gtk_menu_shell_append (GTK_MENU_SHELL (submenu), menuitem);
+	gtk_widget_show (menuitem);
 
 	return menu;
 }
@@ -1047,8 +1027,9 @@ drag_begin_menu_cb (GtkWidget *widget, GdkDragContext     *context)
 {
 	/* FIXME: workaround for a possible gtk+ bug
 	 *    See bugs #92085(gtk+) and #91184(panel) for details.
+	 *    Maybe it's not needed with GtkTooltip?
 	 */
-	gtk_tooltips_disable (panel_tooltips);
+	g_object_set (widget, "has-tooltip", FALSE, NULL);
 }
 
 /* This is a _horrible_ hack to have this here. This needs to be added to the
@@ -1068,8 +1049,7 @@ drag_end_menu_cb (GtkWidget *widget, GdkDragContext     *context)
   /* FIXME: workaround for a possible gtk+ bug
    *    See bugs #92085(gtk+) and #91184(panel) for details.
    */
-  if (panel_global_config_get_tooltips_enabled ())
-    gtk_tooltips_enable (panel_tooltips);
+  g_object_set (widget, "has-tooltip", TRUE, NULL);
 
   while (parent)
     {
@@ -1202,26 +1182,21 @@ void
 setup_menuitem (GtkWidget   *menuitem,
 		GtkIconSize  icon_size,
 		GtkWidget   *image,
-		const char  *title,
-		gboolean     create_invisible_mnemonic)
+		const char  *title)
 			       
 {
 	GtkWidget *label;
+	char      *_title;
 
-	if (create_invisible_mnemonic) {
-		char *_title;
+	/* this creates a label with an invisible mnemonic */
+	label = g_object_new (GTK_TYPE_ACCEL_LABEL, NULL);
+	_title = menu_escape_underscores_and_prepend (title);
+	gtk_label_set_text_with_mnemonic (GTK_LABEL (label), _title);
+	g_free (_title);
 
-		label = g_object_new (GTK_TYPE_ACCEL_LABEL, NULL);
-		_title = menu_escape_underscores_and_prepend (title);
-		gtk_label_set_text_with_mnemonic (GTK_LABEL (label), _title);
-		g_free (_title);
+	gtk_label_set_pattern (GTK_LABEL (label), "");
 
-		gtk_label_set_pattern (GTK_LABEL (label), "");
-
-		gtk_accel_label_set_accel_widget (GTK_ACCEL_LABEL (label),
-						  menuitem);
-	} else
-		label = gtk_label_new_with_mnemonic (title);
+	gtk_accel_label_set_accel_widget (GTK_ACCEL_LABEL (label), menuitem);
 
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 	gtk_widget_show (label);
@@ -1430,22 +1405,21 @@ create_submenu_entry (GtkWidget          *menu,
 					panel_menu_icon_get_size (),
 					NULL,
 					gmenu_tree_directory_get_icon (directory),
-					"gnome-fs-directory");
+					PANEL_ICON_FOLDER);
 
 	setup_menuitem (menuitem,
 			panel_menu_icon_get_size (),
 			NULL,
-			gmenu_tree_directory_get_name (directory),
-			TRUE);
+			gmenu_tree_directory_get_name (directory));
 
 	if (gmenu_tree_directory_get_comment (directory))
-		gtk_tooltips_set_tip (panel_tooltips,
-				      menuitem,
-				      gmenu_tree_directory_get_comment (directory),
-				      NULL);
+		panel_util_set_tooltip_text (menuitem,
+					     gmenu_tree_directory_get_comment (directory));
 
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
 
+	gtk_widget_show (menuitem);
+
 	return menuitem;
 }
 
@@ -1520,18 +1494,16 @@ create_menuitem (GtkWidget          *menu,
 			panel_menu_icon_get_size (),
 			NULL,
 			alias_directory ? gmenu_tree_directory_get_name (alias_directory) :
-					  gmenu_tree_entry_get_name (entry),
-			TRUE);
+					  gmenu_tree_entry_get_name (entry));
 
 	if ((alias_directory &&
 	     gmenu_tree_directory_get_comment (alias_directory)) ||
 	    (!alias_directory &&
 	     gmenu_tree_entry_get_comment (entry)))
-		gtk_tooltips_set_tip (panel_tooltips,
-				      menuitem,
-				      alias_directory ? gmenu_tree_directory_get_comment (alias_directory) :
-							gmenu_tree_entry_get_comment (entry),
-				      NULL);
+		panel_util_set_tooltip_text (menuitem,
+					     alias_directory ?
+						gmenu_tree_directory_get_comment (alias_directory) :
+						gmenu_tree_entry_get_comment (entry));
 
 	g_signal_connect_after (menuitem, "button_press_event",
 				G_CALLBACK (menuitem_button_press_event), NULL);
@@ -1571,6 +1543,8 @@ create_menuitem (GtkWidget          *menu,
 
 	g_signal_connect (menuitem, "activate",
 			  G_CALLBACK (activate_app_def), entry);
+
+	gtk_widget_show (menuitem);
 }
 
 static void
@@ -1748,14 +1722,13 @@ setup_menu_item_with_icon (GtkWidget   *item,
 			   GtkIconSize  icon_size,
 			   const char  *icon_name,
 			   const char  *stock_id,
-			   const char  *title,
-			   gboolean     create_invisible_mnemonic)
+			   const char  *title)
 {
 	if (icon_name || stock_id)
 		panel_load_menu_image_deferred (item, icon_size,
 						stock_id, icon_name, NULL);
 
-	setup_menuitem (item, icon_size, NULL, title, create_invisible_mnemonic);
+	setup_menuitem (item, icon_size, NULL, title);
 }
 
 static void
